@@ -26,7 +26,7 @@
 #'    fsbrain::download_optional_data();
 #'    subjects_dir = fsbrain::get_optional_data_filepath("subjects_dir");
 #'    cm = vis.subject.morph.native(subjects_dir, 'subject1', 'thickness',
-#'     makecmap_options=list('n'=100));
+#'     makecmap_options=list('n'=100, 'colFn'=viridis::viridis));
 #'    # You could mess with the meshes here.
 #'    vis.coloredmeshes(cm);
 #' }
@@ -93,6 +93,7 @@ vis.coloredmeshes <- function(coloredmeshes, background="white", skip_all_na=TRU
     }
 
     perform.rglactions(rglactions);
+
     return(invisible(coloredmeshes));
 }
 
@@ -214,7 +215,7 @@ vis.coloredmeshes.rotating <- function(coloredmeshes, background="white", skip_a
             rgl::play3d(rgl::spin3d(axis = c(x, y, z), rpm = rpm), duration = duration);
         }
     } else {
-        warning("Cannot show rotating scene with NULL device.");    # nocov
+        message("Cannot show rotating scene with NULL device.");    # nocov
     }
 
     perform.rglactions(rglactions);
@@ -335,7 +336,7 @@ get.rglstyle.parameters <- function(renderable, style) {
 #'
 #' @description Run \code{\link{material3d}} without arguments to see valid style keywords to create new styles.
 #'
-#' @param style string. A style name. Available styles are one of: "default", "shiny", "semitransparent", "edges".
+#' @param style string. A style name. Available styles are one of: "default", "shiny", "semitransparent", "glass", "edges".
 #'
 #' @return a style, resolved to a parameter list compatible with \code{\link{material3d}}.
 #'
@@ -351,6 +352,10 @@ get.rglstyle <- function(style) {
         return(get.rglstyle.edges());
     } else if (style == "semitransparent") {
         return(get.rglstyle.semitransparent());
+    } else if (style == "glass") {
+        return(get.rglstyle.glass());
+    } else if (style == "glass2") {
+        return(get.rglstyle.glass2());
     } else {
         stop(sprintf("No such rendering style: '%s'. Try something like 'default', 'shiny', 'edges' or 'semitransparent'.\n", style));
     }
@@ -371,7 +376,7 @@ get.rglstyle.default <- function() {
 
 #' @title Get the semi-transparent visualization style parameters as a named list.
 #'
-#' @description Semitransparent rendering style. This style has a very negative impact on rendering performance. Hint: Run \code{\link{material3d}} without arguments to see valid style keywords to create new styles.
+#' @description Semitransparent rendering style. This style has a very negative impact on rendering performance (in interactive mode). Hint: Run \code{\link{material3d}} without arguments to see valid style keywords to create new styles.
 #'
 #' @return named list, style parameters that can be passed to \code{\link{shade3d}} via \code{\link{do.call}}.
 #'
@@ -383,7 +388,7 @@ get.rglstyle.semitransparent <- function() {
 
 #' @title Get the mesh edges visualization style parameters as a named list.
 #'
-#' @description Mesh edges rendering style.
+#' @description Mesh edges rendering style. Zoom in enough to see them.
 #'
 #' @return named list, style parameters that can be passed to \code{\link{shade3d}} via \code{\link{do.call}}.
 #'
@@ -393,6 +398,29 @@ get.rglstyle.edges <- function() {
 }
 
 
+#' @title Get the glass visualization style parameters as a named list.
+#'
+#' @description Glass-brain rendering style. This style has a very negative impact on rendering performance (especially in interactive mode). Hint: Run \code{\link{material3d}} without arguments to see valid style keywords to create new styles.
+#'
+#' @return named list, style parameters that can be passed to \code{\link{shade3d}} via \code{\link{do.call}}.
+#'
+#' @keywords internal
+get.rglstyle.glass <- function() {
+    return(list("shininess"=50, specular="black", alpha=0.4, front="filled", back="culled"));
+}
+
+
+#' @title Get the glass2 visualization style parameters as a named list.
+#'
+#' @description Glass-brain rendering style. This style has a very negative impact on rendering performance (especially in interactive mode). Hint: Run \code{\link{material3d}} without arguments to see valid style keywords to create new styles.
+#'
+#' @return named list, style parameters that can be passed to \code{\link{shade3d}} via \code{\link{do.call}}.
+#'
+#' @keywords internal
+get.rglstyle.glass2 <- function() {
+    return(list("shininess"=80, specular="white", alpha=0.4, front="filled", back="culled"));
+}
+
 
 #' @title Get a shiny visualization style.
 #'
