@@ -112,7 +112,23 @@ brainview.si <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
 
     coloredmeshes = shift.hemis.rglactions(coloredmeshes, rglactions);
 
-    return(invisible(vis.coloredmeshes(coloredmeshes, rgloptions = rgloptions, rglactions = rglactions, draw_colorbar = draw_colorbar, style = style)));
+
+    callback_interactive_setup_mesh_before_render = getOption('fsbrain.callback_hook_interactive_setup_mesh_before_render', default=NULL);
+    if(is.function(callback_interactive_setup_mesh_before_render)) {
+        coloredmeshes = callback_interactive_setup_mesh_before_render(coloredmeshes);
+    }
+
+    cm = vis.coloredmeshes(coloredmeshes, rgloptions = rgloptions, rglactions = rglactions, draw_colorbar = draw_colorbar, style = style);
+
+    # At this time, the rglactions have already been performed in vis.coloredmeshes, so the changes will not show up in
+    # a snapshot that the user may have requested to be taken. This is unfortunate and needs to be fixed. A workaround would
+    # be to simply take the snapshort manually (by calling rgl::rgl.snaphost() after this function, which will do what you want since this 'si' view)
+    # only has one rendered view/tile.
+    callback_after_render = getOption('fsbrain.callback_hook_after_render', default=NULL);
+    if(is.function(callback_after_render)) {
+        callback_after_render('interactive');
+    }
+    return(invisible(cm));
 }
 
 
@@ -185,6 +201,31 @@ get.sorted.cmeshes <- function(coloredmeshes) {
 }
 
 
+#' @title Highlight requested points (if any), but apply given view rotation before doing so.
+#'
+#' @param hemi character string, one of 'lh', 'rh' or 'both'. If lh or rh, plots only points from that hemi (if hemi info is available for the points).
+#'
+#' @keywords internal
+handle.rglactions.highlight.points <- function(rglactions, angle_rad, x, y, z, hemi = "both") {
+    if('highlight_points' %in% names(rglactions)) {
+        coords = rglactions$highlight_points$coords;
+        color = rglactions$highlight_points$color;
+        color = recycle(color, nrow(coords));
+        if(hemi != "both") { # limit to current hemi if hemi annotation is available in rglactions.
+            if( ! is.null(rglactions$highlight_points$hemi)) {
+                coords = coords[which(rglactions$highlight_points$hemi == hemi), ];
+                color = color[which(rglactions$highlight_points$hemi == hemi)];
+            }
+        }
+        if(length(color) == 0L | length(coords) == 0L) {
+            # Can happen if the hemi filtering removes all coords (i.e., no points to highlight on this hemi).
+            return(invisible(NULL));
+        }
+        coords = rgl::rotate3d(coords, angle_rad, x, y, z);
+        highlight.points.spheres(coords, color = color, radius = rglactions$highlight_points$radius);
+    }
+}
+
 #' @title Visualize a list of colored meshes from four angles.
 #'
 #' @param coloredmeshes, list of coloredmesh. A coloredmesh is a named list as returned by the coloredmesh.from.* functions. It has the entries 'mesh' of type tmesh3d, a 'col', which is a color specification for such a mesh.
@@ -246,40 +287,58 @@ brainview.t4 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     Sys.sleep(1);
     rgl::bg3d(background);
     rgl::layout3d(layout_mat, widths=layout_column_widths, height=layout_row_heights);
+    callback_after_render = getOption('fsbrain.callback_hook_after_render', default=NULL);
 
     # Create the upper left view: draw only the left hemi, from the left
     rgl::next3d(reuse=TRUE);
     vis.rotated.coloredmeshes(lh_meshes, pi/2, 1, 0, 0, style=style);
+    handle.rglactions.highlight.points(rglactions, pi/2, 1, 0, 0, hemi = "lh");
     rgl::rgl.viewpoint(-90, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"lateral lh");
     }
+    if(is.function(callback_after_render)) {
+        callback_after_render('lateral_lh');
+    }
+
 
     # Create the upper right view
     rgl::next3d(reuse=FALSE);
     vis.rotated.coloredmeshes(rh_meshes, pi/2, 1, 0, 0, style=style);
+    handle.rglactions.highlight.points(rglactions, pi/2, 1, 0, 0, hemi = "rh");
     rgl::rgl.viewpoint(90, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"lateral rh");
     }
+    if(is.function(callback_after_render)) {
+        callback_after_render('lateral_rh');
+    }
 
 
     # Create the lower left view
     rgl::next3d(reuse=FALSE);
     vis.rotated.coloredmeshes(lh_meshes, pi/2, 1, 0, 0, style=style);
+    handle.rglactions.highlight.points(rglactions, pi/2, 1, 0, 0, hemi = "lh");
     rgl::rgl.viewpoint(90, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"medial lh");
     }
+    if(is.function(callback_after_render)) {
+        callback_after_render('medial_lh');
+    }
 
 
     # Create the lower right view
     rgl::next3d(reuse=FALSE);
     vis.rotated.coloredmeshes(rh_meshes, pi/2, 1, 0, 0, style=style);
+    handle.rglactions.highlight.points(rglactions, pi/2, 1, 0, 0, hemi = "rh");
     rgl::rgl.viewpoint(-90, 0, fov=0, interactive=FALSE);
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"medial rh");
     }
+    if(is.function(callback_after_render)) {
+        callback_after_render('medial_rh');
+    }
 
     if(is.character(draw_colorbar)) {
         rgl::next3d(reuse=FALSE);
@@ -289,11 +348,14 @@ brainview.t4 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     }
 
 
-    perform.rglactions(rglactions);
+    perform.rglactions(rglactions, ignore = c("highlight_points"));
     return(invisible(coloredmeshes));
 }
 
 
+
+
+
 #' @title Visualize a list of colored meshes from nine angles.
 #'
 #' @param coloredmeshes, list of coloredmesh. A coloredmesh is a named list as returned by the coloredmesh.from.* functions. It has the entries 'mesh' of type tmesh3d, a 'col', which is a color specification for such a mesh.
@@ -325,10 +387,6 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
         stop("Parameter 'coloredmeshes' must be a list.");
     }
 
-    layout_dim_x = 3;
-    layout_dim_y = 3;
-    num_views = layout_dim_x * layout_dim_y;
-
     sorted_meshes = get.sorted.cmeshes(coloredmeshes);
     lh_meshes = sorted_meshes$lh;
     rh_meshes = sorted_meshes$rh;
@@ -363,7 +421,7 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     rgl::layout3d(layout_mat, widths=layout_column_widths, height=layout_row_heights);
 
     coloredmeshes_potentially_shifted = shift.hemis.rglactions(coloredmeshes, rglactions);
-
+    callback_after_render = getOption('fsbrain.callback_hook_after_render', default=NULL);
 
     #  ------------------ Row 1 --------------------
 
@@ -375,6 +433,9 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"lateral lh");
     }
+    if(is.function(callback_after_render)) {
+        callback_after_render('lateral_lh');
+    }
 
     # Create the upper central view: draw both hemis from above (top view)
     rgl::next3d(reuse=FALSE);
@@ -383,6 +444,9 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     if(draw_labels) {
         rgl::text3d(0,label_shift_y_dorsal,0,"dorsal");
     }
+    if(is.function(callback_after_render)) {
+        callback_after_render('dorsal');
+    }
 
     # Create the upper right view
     rgl::next3d(reuse=FALSE);
@@ -391,6 +455,9 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"lateral rh");
     }
+    if(is.function(callback_after_render)) {
+        callback_after_render('lateral_rh');
+    }
 
 
     #  ------------------ Row 2 --------------------
@@ -403,6 +470,9 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"medial lh");
     }
+    if(is.function(callback_after_render)) {
+        callback_after_render('medial_lh');
+    }
 
     # Create the 2nd row central view: draw both hemis from below (bottom view)
     rgl::next3d(reuse=FALSE);
@@ -411,6 +481,9 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     if(draw_labels) {
         rgl::text3d(0,label_shift_y_ventral,0,"ventral");
     }
+    if(is.function(callback_after_render)) {
+        callback_after_render('ventral');
+    }
 
 
     # Create the 2nd row right view
@@ -420,6 +493,9 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"medial rh");
     }
+    if(is.function(callback_after_render)) {
+        callback_after_render('medial_rh');
+    }
 
 
     #  ------------------ Row 3 --------------------
@@ -432,6 +508,9 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"rostral");
     }
+    if(is.function(callback_after_render)) {
+        callback_after_render('rostral');
+    }
 
     # Create the bottom central view.
     rgl::next3d(reuse=FALSE);
@@ -446,6 +525,9 @@ brainview.t9 <- function(coloredmeshes, background="white", skip_all_na=TRUE, st
     if(draw_labels) {
         rgl::text3d(0,label_shift_y,0,"caudal");
     }
+    if(is.function(callback_after_render)) {
+        callback_after_render('caudal');
+    }
 
     if(is.character(draw_colorbar)) {
         rgl::next3d(reuse=FALSE);
@@ -527,32 +609,48 @@ brainview.sd <- function(coloredmeshes, view_angle, background="white", skip_all
         rgl::layout3d(layout_mat, widths=layout_column_widths, height=layout_row_heights);
         rgl::next3d(reuse=TRUE);
     }
-
+    callback_after_render = getOption('fsbrain.callback_hook_after_render', default=NULL);
 
     if(view_angle == "lateral_lh") {
         vis.rotated.coloredmeshes(lh_meshes, pi/2, 1, 0, 0, style=style);
+        handle.rglactions.highlight.points(rglactions, pi/2, 1, 0, 0, hemi = "lh");
         rgl::rgl.viewpoint(-90, 0, fov=0, interactive=FALSE);
+        if(is.function(callback_after_render)) { callback_after_render('lateral_lh'); }
     } else if (view_angle == "dorsal") {
         vis.rotated.coloredmeshes(coloredmeshes, 0, 1, 0, 0, style=style);
+        handle.rglactions.highlight.points(rglactions, 0, 1, 0, 0, hemi = "both");
         rgl::rgl.viewpoint(0, 0, fov=0, interactive=FALSE);
+        if(is.function(callback_after_render)) { callback_after_render('dorsal'); }
     } else if(view_angle == "lateral_rh") {
         vis.rotated.coloredmeshes(rh_meshes, pi/2, 1, 0, 0, style=style);
+        handle.rglactions.highlight.points(rglactions, pi/2, 1, 0, 0, hemi = "rh");
         rgl::rgl.viewpoint(90, 0, fov=0, interactive=FALSE);
+        if(is.function(callback_after_render)) { callback_after_render('lateral_lh'); }
     } else if(view_angle == "medial_lh") {
         vis.rotated.coloredmeshes(lh_meshes, pi/2, 1, 0, 0, style=style);
+        handle.rglactions.highlight.points(rglactions, pi/2, 1, 0, 0, hemi = "lh");
         rgl::rgl.viewpoint(90, 0, fov=0, interactive=FALSE);
+        if(is.function(callback_after_render)) { callback_after_render('medial_lh'); }
     } else if(view_angle == "ventral") {
         vis.rotated.coloredmeshes(coloredmeshes, pi, 1, 0, 0, style=style);
+        handle.rglactions.highlight.points(rglactions, pi/2, 1, 0, 0, hemi = "both");
         rgl::rgl.viewpoint(0, 0, fov=0, interactive=FALSE);
+        if(is.function(callback_after_render)) { callback_after_render('ventral'); }
     } else if(view_angle == "medial_rh") {
         vis.rotated.coloredmeshes(rh_meshes, pi/2, 1, 0, 0, style=style);
+        handle.rglactions.highlight.points(rglactions, pi/2, 1, 0, 0, hemi = "rh");
         rgl::rgl.viewpoint(-90, 0, fov=0, interactive=FALSE);
+        if(is.function(callback_after_render)) { callback_after_render('medial_rh'); }
     } else if(view_angle == "rostral") {
         vis.rotated.coloredmeshes(coloredmeshes, pi/2, 1, 0, 0, style=style);
+        handle.rglactions.highlight.points(rglactions, pi/2, 1, 0, 0, hemi = "both");
         rgl.viewpoint(0, 0, fov=0, interactive=FALSE);
+        if(is.function(callback_after_render)) { callback_after_render('rostral'); }
     } else if(view_angle == "caudal") {
         vis.rotated.coloredmeshes(coloredmeshes, pi/2, 1, 0, 0, style=style);
+        handle.rglactions.highlight.points(rglactions, pi/2, 1, 0, 0, hemi = "both");
         rgl::rgl.viewpoint(180, 0, fov=0, interactive=FALSE);
+        if(is.function(callback_after_render)) { callback_after_render('caudal'); }
     } else {
         stop(sprintf("Invalid view_angle '%s'. Must be one of 'lateral_lh', 'dorsal', 'lateral_rh', 'medial_lh', 'ventral', 'medial_rh', 'rostral' or 'caudal'.\n", view_angle));
     }
@@ -562,7 +660,7 @@ brainview.sd <- function(coloredmeshes, view_angle, background="white", skip_all
         draw.colorbar(coloredmeshes, horizontal=horizontal);
     }
 
-    perform.rglactions(rglactions);
+    perform.rglactions(rglactions, ignore = c('highlight_points'));
 
     return(invisible(coloredmeshes));
 }
@@ -640,13 +738,13 @@ shift.hemis.apart <- function(coloredmeshes_hl, shift_by=NULL, axis=1L, hemi_ord
         }
 
         lh_src_mesh$vertices[,axis] = lh_src_mesh$vertices[,axis] + shift_by[1];
-        lh_new_tmesh = rgl::tmesh3d(c(t(lh_src_mesh$vertices)), c(t(lh_src_mesh$faces)), homogeneous=FALSE);
+        lh_new_tmesh = fs.surface.to.tmesh3d(lh_src_mesh);
         coloredmeshes_hl$lh$mesh$vb = lh_new_tmesh$vb;
         coloredmeshes_hl$lh$mesh$it = lh_new_tmesh$it;
 
 
         rh_src_mesh$vertices[,axis] = rh_src_mesh$vertices[,axis] + shift_by[2];
-        rh_new_tmesh = rgl::tmesh3d(c(t(rh_src_mesh$vertices)), c(t(rh_src_mesh$faces)), homogeneous=FALSE);
+        rh_new_tmesh = fs.surface.to.tmesh3d(rh_src_mesh);
         coloredmeshes_hl$rh$mesh$vb = rh_new_tmesh$vb;
         coloredmeshes_hl$rh$mesh$it = rh_new_tmesh$it;
     } else {
